﻿-- Author      : RisM
-- Create Date : 6/28/2009 4:04:27 PM

SpeakinSpell = LibStub("AceAddon-3.0"):GetAddon("SpeakinSpell")
local L	= LibStub("AceLocale-3.0"):GetLocale("SpeakinSpell", false)

-------------------------------------------------------------------------------
-- RANDOM SPEECHES - THE CORE FUNCTIONALITY
-------------------------------------------------------------------------------


function SpeakinSpell:OnEventDetected( DetectedEventStub )
	local funcname = "OnEventDetected"

	-- fill in the stub
	local DetectedEvent = self:CreateDetectedEvent( DetectedEventStub )
	
	local rankedkey, ranklesskey = self:MakeEventTableKeys( DetectedEvent )
	
	local SpeakForRankless = true
	
	--NOTE: it looks like I want to make a subroutine here and call it twice
	--		but the logic is actually slightly different between the ranked and rankless cases
	
	-- look for an event table entry for the ranked version
	if rankedkey then
		local de = self:CopyTable( DetectedEvent )
		de.key = rankedkey
		de.EventTableEntry = SpeakinSpellSavedData.EventTable[ de.key ]
		
		if SpeakinSpellSavedData.ShowSpellEvents then
			if de.rank ~= "" then
				self:Print( self:ProcessSubstitutionVars(L.EVENTTYPES.IN_SPELL_LIST[de.type]..de.SpellLink.." ("..de.rank..")") ) --TODOLATER: add <target> and consider <displaylink> to encapsulate "L.EVENTTYPES.IN_SPELL_LIST[de.type]..de.SpellLink"
			else
				self:Print( self:ProcessSubstitutionVars(L.EVENTTYPES.IN_SPELL_LIST[de.type]..de.SpellLink) ) --TODOLATER: add <target> and consider <displaylink> to encapsulate "L.EVENTTYPES.IN_SPELL_LIST[de.type]..de.SpellLink"
			end
		end

		if de.EventTableEntry then
			SpeakForRankless = false
			if self:AllowSpeakForSpell( de ) then
				self:SpeakForSpell( de )
			end
		else
			-- record a new event for this ranked key
			self:DebugMsg(funcname, "no EventTableEntry for key:"..tostring(de.key))
			self:RecordNewEvent( de )
		end
	end
	
	-- check for a rankless version, but don't speak for both
	if ranklesskey then
		local de = self:CopyTable( DetectedEvent )
		de.key = ranklesskey
		de.EventTableEntry = SpeakinSpellSavedData.EventTable[ de.key ]
		
		if SpeakinSpellSavedData.ShowSpellEvents then
			self:Print( self:ProcessSubstitutionVars(L.EVENTTYPES.IN_SPELL_LIST[de.type]..de.SpellLink.." "..L["(Any Rank)"]) ) --TODOLATER add <target> and consider <displaylink> to encapsulate "L.EVENTTYPES.IN_SPELL_LIST[de.type]..de.SpellLink"
		end
		
		if SpeakinSpellSavedData.EventTable[ de.key ] then
			if SpeakForRankless then
				if self:AllowSpeakForSpell( de ) then
					self:SpeakForSpell( de )
				end
			end
		else
			-- record a new RANKLESS event for this rankless key
			de.rank = "" -- to make the new event rankless -- NOTE: don't erase this before speaking, to maintain validity of <spellrank> variable
			de.DisplayName = self:FormatDisplayName( de ) -- update display name to show "any rank"
			self:DebugMsg(funcname, "no EventTableEntry for key:"..tostring(de.key))
			self:RecordNewEvent(de)
		end
	end

end


	
function SpeakinSpell:SelectRandomMessage(DetectedEvent)
	assert(DetectedEvent.EventTableEntry and DetectedEvent.EventTableEntry.Messages) -- this function should not be called unless we have a valid table for this spell
	
	-- select a random message from the user-specific table
	
	local sizelist = #(DetectedEvent.EventTableEntry.Messages)
	local tempnum = math.random(1, sizelist);
	
	msg = DetectedEvent.EventTableEntry.Messages[tempnum]
	assert(msg)
	assert(type(msg) == "string")
	
	-- avoid repeating the same message we used last time
	-- unless there's only one in the list
	local History = self.RuntimeData.AnnouncementHistory[DetectedEvent.key]
	if History then
		-- we've announced this event before
		-- make sure we select a different speech than last time, if applicable
		while (sizelist > 1) and (msg == History.LastMessage) do
			tempnum = math.random(1, sizelist);
			msg = DetectedEvent.EventTableEntry.Messages[tempnum]
		end
	end
	
	return msg
end


function SpeakinSpell:IsInWintergraspBattle()
	--only applies to active battles
	
	--we only care if a battle is in progress
	--if we're in WG for VOA PVE and the battle is over, switch to raid settings
	if (GetWintergraspWaitTime() ~= nil) then
		-- a battle is not in progress
		return false
	end
	
	--TODOFUTURE: this has a localization problem since it does a string comparison
	-- unfortunately there doesn't appear to be a locale-independant way to test for wintergrasp
	if GetZoneText() == L["Wintergrasp GetZoneText"] then
		return true
	else
		return false
	end
end


function SpeakinSpell:GetChatChannelForSpell(EventTableEntry)
	local funcname = "GetChatChannelForSpell"
	
	-- NOTE: the EventTableEntry.Channels table stores channel names that are compatible with SendChatMessage
	--	see also CurrentMessagesGUI_OnChannelSelect
	if (IsActiveBattlefieldArena()) then
		return EventTableEntry.Channels.Arena
	elseif (GetNumBattlefieldScores() > 0) then
		return EventTableEntry.Channels.BG
	elseif (self:IsInWintergraspBattle()) then --only applies to active battles
		-- NOTE: raid warnings not enabled for WG battles
		return EventTableEntry.Channels.WG
	elseif (GetNumRaidMembers() > 0) then
		local RaidChannel = EventTableEntry.Channels.Raid

		-- if the player wants to speak in the RAID_WARNING channel, 
		-- but doesn't have permission (not the leader or assist)
		-- then fall back to the RAID channel instead, so we say something at all
		-- instead of triggering an error message in the chat
		if RaidChannel == "RAID_WARNING" then
			if not (IsRaidOfficer() or IsRaidLeader()) then
				self:DebugMsg(funcname, "You don't have permission to say Raid Warnings right now - using RAID chat instead")
				RaidChannel = "RAID"
			end
		end
		
		return RaidChannel
	elseif (GetNumPartyMembers() > 0) then
		return EventTableEntry.Channels.Party
	else -- solo
		return EventTableEntry.Channels.Solo
	end
end


function SpeakinSpell:GetLanguageList()
	--TODO: use this function somewhere
	--		this is just some sample code captured from wowwiki while I'm looking at it
	
	--Create and return a list of RP languages that are known to the player
	Langs = {}
	for i=1, GetNumLanguages() do
		tinsert(Langs, GetLanguageByIndex(i))
	end
	return Langs
end


function SpeakinSpell:InsertRandomFaction(name)
	if (name ~= nil) and (name ~= "") then
		tinsert(L.RANDOM["randomfaction"], name)
	end
end


function SpeakinSpell:AddRandomFactions()
	-- add the player's guild name (if any)
	-- guildName, guildRankName, guildRankIndex = GetGuildInfo(unit);
	local guildName = GetGuildInfo( UnitName("player") )
	self:InsertRandomFaction(guildName)
	
	-- add the player's arena teams (if any)
	-- teamName, teamSize, teamRating, weekPlayed, weekWins, seasonPlayed, seasonWins, playerPlayed, seasonPlayerPlayed, teamRank, playerRating = GetArenaTeam(id);
	local teamName2 = GetArenaTeam(1);
	local teamName3 = GetArenaTeam(2);
	local teamName5 = GetArenaTeam(5);

	self:InsertRandomFaction(teamName2)
	self:InsertRandomFaction(teamName3)
	self:InsertRandomFaction(teamName5)
end


-- returns a random entry from the L.RANDOM[something] table
-- for example L.RANDOM["randomfaction"] is a list of faction names
function SpeakinSpell:GetRandomLocaleText(key)
	local t = L.RANDOM[key]
	local sizelist = #(t)
	local tempnum = math.random(1, sizelist);
	return t[tempnum]
end


function SpeakinSpell:DoSubstitution(var, DetectedEvent)
	
	-- NOTE: also support the english variable names if running in a non-english locale
	--		this is meant to help support the fact that documentation in-game is lacking for this variable list
	--		it's also standard for non-English clients to support English commands

	local Substitutions = {
		[                "player"   ] = UnitName("player"), -- the player's name / your name
		[string.lower(L["<player>"])] = UnitName("player"), -- the player's name / your name
		
		[                "caster"   ] = DetectedEvent.caster, -- the caster's name
		[string.lower(L["<caster>"])] = DetectedEvent.caster, -- the caster's name
		
		[                "target"   ] = DetectedEvent.target, -- the target's name
		[string.lower(L["<target>"])] = DetectedEvent.target, -- the target's name
		
		[                "focus"   ] = UnitName("focus"), -- the focus target's name
		[string.lower(L["<focus>"])] = UnitName("focus"), -- the focus target's name
		
		[                "targetclass"   ] = DetectedEvent.targetclass, -- the target's name
		[string.lower(L["<targetclass>"])] = DetectedEvent.targetclass, -- the target's name
		
		[                "targetrace"   ] = DetectedEvent.targetrace, -- the target's name
		[string.lower(L["<targetrace>"])] = DetectedEvent.targetrace, -- the target's name
		
		[                "spellname"   ] = DetectedEvent.name,
		[string.lower(L["<spellname>"])] = DetectedEvent.name,
		
		[                "spellrank"   ] = DetectedEvent.rank,
		[string.lower(L["<spellrank>"])] = DetectedEvent.rank,
		
		[                "spelllink"   ] = DetectedEvent.SpellLink,
		[string.lower(L["<spelllink>"])] = DetectedEvent.SpellLink,
		
		[                "pet"   ] = DetectedEvent.pet,
		[string.lower(L["<pet>"])] = DetectedEvent.pet,
		
		[                "randomfaction"   ] = self:GetRandomLocaleText("randomfaction"),
		[string.lower(L["<randomfaction>"])] = self:GetRandomLocaleText("randomfaction"),
		
		[                "randomtaunt"   ] = self:GetRandomLocaleText("randomtaunt"),
		[string.lower(L["<randomtaunt>"])] = self:GetRandomLocaleText("randomtaunt"),
		
		[                "realm"   ] = GetRealmName(),
		[string.lower(L["<realm>"])] = GetRealmName(),

		-- GetRealZoneText is subtly different from GetZoneText re: instance names
		-- see http://www.wowwiki.com/API_GetZoneText
		-- we want to use GetRealZoneText to use real instance names instead of subzone names
		-- <subzone> will use the minimap text for instances
		[                "zone"   ] = GetRealZoneText(),
		[string.lower(L["<zone>"])] = GetRealZoneText(),

		-- GetSubZoneText returns an empty string if you're not in a sub zone
		-- GetMinimapZoneText returns the current subzone text shown on the minimap
		-- the minimap always shows something = the greater region name if not in a subzone
		-- so this is guaranteed to return an appropriate value
		[                "subzone"   ] = GetMinimapZoneText(),
		[string.lower(L["<subzone>"])] = GetMinimapZoneText(),

		[                "playertitle"   ] = GetTitleName( GetCurrentTitle() ),
		[string.lower(L["<playertitle>"])] = GetTitleName( GetCurrentTitle() ),

		-- auto-substitutions like MS Word would use
		[                "TM"   ] = "™",	-- Trademark symbol
		[string.lower(L["<TM>"])] = "™",	-- Trademark symbol
		
		[                "R"   ] = "®",	-- Registered Trademark symbol
		[string.lower(L["<R>"])] = "®",	-- Registered Trademark symbol
		
		[                "C"   ] = "©",	-- Copyright (C)
		[string.lower(L["<C>"])] = "©",	-- Copyright (C)
	}

	-- do case insensitive lookup of variable names
	local varkey = string.lower(var)

	-- check for var = "<target|___>", or localized equivalent
	-- this should normally work the same as <target>
	-- however if the target is the player, then show ___ instead of the player's own name
	-- For example "Hey no hitting on <target|me>!" results in
	-- "Hey no hitting on Fred!" if cast on Fred
	-- "Hey no hitting on me!" if cast on yourself

	local ndxPipe = string.find(varkey, "|")
	if ndxPipe and (ndxPipe >= 0) then
		varkey = string.sub(varkey, 0, ndxPipe-1)
		local me = string.sub(var, ndxPipe+1) -- NOTE: use original Case Sensitive var name to get "me"
		-- behavior of string.sub seems inconsistent, sometimes ndxPipe+1 gets the | character, which then throws a LUA error
		-- but ndxPipe+2 sometimes misses the first letter of "me"
		-- so we use ndxPipe+1 and check for the leading |
		if string.find(me,"|") then
			me = string.sub(me, 2)
		end
		-- use "me" instead of my own name if targetting myself
		if (me ~= "") then
			-- <target|me>
			if ( (varkey == "target") or (varkey == L["<target>"]) ) and (DetectedEvent.target == UnitName("player")) then
				Substitutions[varkey] = me
			end
			-- <caster|I>
			if ( (varkey == "caster") or (varkey == L["<caster>"]) ) and (DetectedEvent.caster == UnitName("player")) then
				Substitutions[varkey] = me
			end
		end

	end

	-- perform the substitution
	if Substitutions[varkey] then
		return Substitutions[varkey]
	end

	-- if we got this far then we did not find a value for this variable
	-- so return the variable unchanged, with wrapping < > restored
	return "<"..var..">"
end


function SpeakinSpell:ProcessSubstitutionVars(msg, DetectedEvent)
	return string.gsub(msg, "<(.-)>", 
		function(var) 
			return self:DoSubstitution(var, DetectedEvent) 
		end
	)
end


-- return true if this is a slash command
-- return false if this is other default text
function SpeakinSpell:ProcessChatSlashCommand(text)
	local funcname = "ProcessChatSlashCommand"
	
	if ( strsub(text, 1, 1) ~= "/" ) then
		return false
	end
	
	self:DebugMsg(funcname,"executing slash command: " .. text)
	
	-- issue a warning if this a secure command like /cast
	-- If the string is in the format "/cmd blah", command will be "/cmd"
	local command = strmatch(text, "^(/[^%s]+)") or "";
	if IsSecureCmd(command) then
		self:Print(string.format(L["WARNING: can't execute protected command: %s"],text))
		return true
	end
	
	-- create a hidden edit box to parse and send the slash command
	if not SpeakinSpellTempEditBox then
		SpeakinSpellTempEditBox = CreateFrame("EditBox", "SpeakinSpellTempEditBox", UIParent)
		SpeakinSpellTempEditBox:Hide()
	end
	
	-- send the command
	SpeakinSpellTempEditBox:SetText(text) 
	ChatEdit_SendText(SpeakinSpellTempEditBox)
	
	return true
end



function SpeakinSpell:SplitLines(msg)
	local s1 = string.gsub(msg, "<newline>", "\n")
	local s2 = string.gsub(s1, L["<newline>"], "\n")
	local t = { strsplit("\n",s2) }
	return t
end


function SpeakinSpell:SayOneLine(line, channel, WhisperTarget, DetectedEvent)
	-- if this message is a slash command, send it through an editBox function to parse and execute the command
	--NOTE: a specified slash command overrides the default channel and whisper features below
	--		it doesn't make sense to combine them
	if self:ProcessChatSlashCommand(line) then
		return
	end

	-- say the message to the chat, if applicable
	if channel then
		if channel == "SPEAKINSPELL CHANNEL" then
			self:Print(line)
		else
			SendChatMessage(line, channel);
		end
	end
	
	-- send the message to a whisper to the target, if applicable
	if WhisperTarget then
		SendChatMessage(line, "WHISPER", nil, DetectedEvent.target);
	end
end


function SpeakinSpell:SpeakForSpell(DetectedEvent)
	local funcname = "SpeakForSpell"
	
	-- determine which channel we will speak in
	channel = self:GetChatChannelForSpell(DetectedEvent.EventTableEntry)
	
	--TODOFUTURE: make it an option to whisper yourself - but be wary of the auto self-cast logic
	local WhisperTarget = DetectedEvent.EventTableEntry.WhisperTarget and DetectedEvent.target and (DetectedEvent.target ~= "") and (DetectedEvent.target ~= "Unknown") and (DetectedEvent.target ~= UnitName("player")) and UnitIsFriend("player",DetectedEvent.target)

	if not channel and not WhisperTarget then
		-- this spell's speeches have no target for the current situation
		-- NOTE: this rule applies even if a slash command override is present in the randomly selected message text
		self:DebugMsg(funcname,"current scenario has no message targets for " .. DetectedEvent.SpellLink)
		return
	end
	
	local msg = self:SelectRandomMessage(DetectedEvent)
	
	-- record the time when we spoke for this spell, along with the last target, and last message spoken
	-- to support duplicate prevent, announcement limits, and cooldowns
	-- NOTE: record the unprocessed message text, so that changing the value of substitution variables
	--		does not make us think it's a new random message selection
	self:SaveAnnouncementHistory(DetectedEvent, msg)
	
	-- process substitution variables
	msg = self:ProcessSubstitutionVars(msg, DetectedEvent)

	-- break up the message into separate lines, if applicable
	local lines = self:SplitLines(msg)

	for _,line in ipairs(lines) do
		self:SayOneLine(line, channel, WhisperTarget, DetectedEvent)
	end
end

