﻿-- Author      : RisM
-- Create Date : 6/28/2009 4:02:20 PM

SpeakinSpell = LibStub("AceAddon-3.0"):GetAddon("SpeakinSpell")
local L	= LibStub("AceLocale-3.0"):GetLocale("SpeakinSpell", false)



function SpeakinSpell:RegisterAllEvents()
	-- register for spellcasting events
	-- which is our hook too know when to speak for a spell
	-- among other things
	
	-- startup and loading events
	self:RegisterEvent("VARIABLES_LOADED")
	
	-- combat events
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	self:RegisterEvent("UNIT_SPELLCAST_SENT")
	
	-- enter and exit combat
	-- NOTE: PLAYER_ENTER_COMBAT and PLAYER_LEAVE_COMBAT are not what they sound like
	--		Those 2 events are auto-attack on/off notifications
	--		To detect when the combat flag goes on and off from getting/losing aggro
	--		which is what we generally mean by "entering combat" and "exiting combat"
	--		we must check for Regen enabled/disabled, which is our only valid notification
	--		see also: http://www.wowwiki.com/Events/Combat
	self:RegisterEvent("PLAYER_REGEN_DISABLED")
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
	
	-- change zones --
	--self:RegisterEvent("MINIMAP_ZONE_CHANGED") --for subzone changes - does not appear to be needed
	self:RegisterEvent("ZONE_CHANGED_INDOORS") --for subzone changes inside an instance
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA") --for large scale zone changes
	self:RegisterEvent("ZONE_CHANGED") --covers both small and large scale zone changes?
end


---------------------------------------------------------------------------
-- STARTUP AND LOADING EVENTS
---------------------------------------------------------------------------


function SpeakinSpell:VARIABLES_LOADED()
	local funcname = "VARIABLES_LOADED"
	self:DebugMsg(funcname, "Variables Loaded")
	
	-- SpeakinSpellSavedData will always exist
	-- SpeakinSpellSavedData.EventTable only exists if actual data has been loaded
	-- NOTE: SpeakinSpellSavedData.EventTable may be empty, but its parent table SpeakinSpellSavedData never will be
	if	SpeakinSpellSavedData.EventTable or -- current name
		SpeakinSpellSavedData.SpellTable then --old name from v3.1.2.04 and earlier
		-- If the format of the saved data has changed due to upgrading to a newer version
		-- then fix the old saved data for compatibility
		self:FixOldSavedData()
	else
		self:DebugMsg(funcname, "EventTable does not exist, using defaults")
		-- new user: load defaults
		self:InitDefaultSavedData()
	end

	-- set up runtime data that we don't save, only after the saved variables are loaded
	self:InitRuntimeData()
	
	-- Add guild and arena teams to the <randomfaction> substitution
	-- NOTE: don't do this during InitRuntimeData because that is also done when resetting defaults
	--		and we only need this to happen once after loading the addon into memory
	self:AddRandomFactions()
	
	-- set up the Options GUI config - we need the Runtime Data before we can do this
	self:SetupOptionsGUI()
	
	-- trigger a Login event
	-- this can be used for random messages
	self:DebugMsg(funcname, "Signal System Event: SpeakinSpell Loaded")
	local DetectedEventStub = {
		name = L.EVENTNAMES["SpeakinSpell Loaded"],
		rank = "",
		type = "EVENT",
		-- treat these special events as cast by the player on himself
		caster = UnitName("player"),
		target = UnitName("player"),
	}
	self:OnEventDetected( DetectedEventStub ) 
	
	-- if FixOldSavedData wants the user to review the updated data
	-- show the requested options frame now
	if self.ShowSettingsPageOnLoad then
		InterfaceOptionsFrame_OpenToCategory(self.ShowSettingsPageOnLoad)
		self.ShowSettingsPageOnLoad = nil
	end
	
end



---------------------------------------------------------------------------
-- COMBAT EVENTS
---------------------------------------------------------------------------


function SpeakinSpell:SPELL_AURA_APPLIED(timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, spellId, spellName, spellSchool, auraType)
	local funcname = "SPELL_AURA_APPLIED"
	self:DebugMsg(funcname, "( srcName, dstName, auraType, spellName )")
	self:DebugMsgDumpString("srcName",srcName)
	self:DebugMsgDumpString("dstName",dstName)
	self:DebugMsgDumpString("auraType",auraType)
	self:DebugMsgDumpString("spellName",spellName)
	
	-- we're only concerned with buffs we receive
	if dstName ~= UnitName("player") then
		self:DebugMsg(funcname, "not cast on you")
		return
	end
	
	local DetectedEventStub = {
		name = spellName,
		rank = "", -- no rank known
		-- treat these special events as cast by the player on himself
		caster = srcName,
		target = dstName,
		type = "SPELL_AURA_APPLIED_"..auraType,
		SpellLink = true,
	}
	
	-- buffs cast on me by someone other than me are treated differently than buffs I cast on myself
	if srcName ~= UnitName("player") then
		self:DebugMsg(funcname, "not cast by you")
		DetectedEventStub.type = DetectedEventStub.type.."_FOREIGN"
	end
	
	-- process the spell
	-- this function is shared with the UNIT_SPELLCAST_SENT event handler
	self:OnEventDetected( DetectedEventStub )
end


function SpeakinSpell:COMBAT_LOG_EVENT_UNFILTERED(_,timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, ...)
	--self:DebugMsg("COMBAT_LOG_EVENT_UNFILTERED",tostring(eventtype))
	if eventtype == "SPELL_AURA_APPLIED" then
		self:SPELL_AURA_APPLIED(timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, ...)
	end
end


function SpeakinSpell:UNIT_SPELLCAST_SENT(event, caster, name, rank, target)
	local funcname = "UNIT_SPELLCAST_SENT"

	self:DebugMsg(funcname,"(caster, name, rank, target)")
	--self:DebugMsgDumpString("event",event) -- just echoes UNIT_SPELLCAST_SENT every time
	self:DebugMsgDumpString("caster",caster)
	self:DebugMsgDumpString("name",name)
	self:DebugMsgDumpString("rank",rank)
	self:DebugMsgDumpString("target",target)
	
	-- only speak for spells cast by the player, not anyone else
	if caster == "player" then
		caster = UnitName("player")
	else
		self:DebugMsg(funcname,"caster is not player")
		return
	end
	
	-- process the spell
	local DetectedEventStub = {
		name = name,
		rank = rank,
		caster = caster,
		target = target,
		type = "UNIT_SPELLCAST_SENT",
		SpellLink = true,
	}
	self:OnEventDetected( DetectedEventStub )
end



---------------------------------------------------------------------------
-- ENTERING AND EXITING COMBAT
---------------------------------------------------------------------------
-- NOTE: PLAYER_ENTER_COMBAT and PLAYER_LEAVE_COMBAT are not what they sound like
--		Those 2 events are auto-attack on/off notifications
--		To detect when the combat flag goes on and off from getting/losing aggro
--		which is what we generally mean by "entering combat" and "exiting combat"
--		we must check for Regen enabled/disabled, which is our only valid notification
--		see also: http://www.wowwiki.com/Events/Combat


function SpeakinSpell:PLAYER_REGEN_DISABLED()
	local funcname = "PLAYER_REGEN_DISABLED"

	-- update combat status flags for "limit once per combat" feature
	self.RuntimeData.InCombat = true
	self:ResetOncePerCombatFlags()

	-- Signal the Entering Combat event
	self:DebugMsg(funcname, "Signal System Event: Entering Combat")
	local DetectedEventStub = {
		name = L.EVENTNAMES["Entering Combat"],
		rank = "",
		type = "EVENT",
		-- treat these special events as cast by the player on her default target
		caster = UnitName("player"),
		target = self:GetDefaultTarget(),
	}
	self:OnEventDetected( DetectedEventStub ) 
end


function SpeakinSpell:PLAYER_REGEN_ENABLED()
	local funcname = "PLAYER_REGEN_ENABLED"

	-- update combat status flags for "limit once per combat" feature
	self.RuntimeData.InCombat = false
	self:ResetOncePerCombatFlags()
	
	-- Signal the Exiting Combat event
	self:DebugMsg(funcname, "Signal System Event: Exiting Combat")
	local DetectedEventStub = {
		name = L.EVENTNAMES["Exiting Combat"],
		rank = "",
		type = "EVENT",
		-- treat these special events as cast by the player on her default target
		caster = UnitName("player"),
		target = self:GetDefaultTarget(),
	}
	self:OnEventDetected( DetectedEventStub ) 
end


---------------------------------------------------------------------------
-- MOVING TO A DIFFERENT ZONE OR REGION
---------------------------------------------------------------------------


function SpeakinSpell:OnZoneChange(minoronly)
	local funcname = "OnZoneChange"

	-- Signal the zone change event
	self:DebugMsg(funcname, "Signal System Event: Zone Changed, minoronly="..tostring(minoronly))
	local DetectedEventStub = {
		name = L.EVENTNAMES["Zone Changed"],
		rank = "",
		type = "EVENT",
		-- treat these special events as cast by the player on himself
		caster = UnitName("player"),
		target = UnitName("player"),
	}
	
	if minoronly then
		DetectedEventStub.name = L.EVENTNAMES["Subzone Changed"]
	end
	
	self:OnEventDetected( DetectedEventStub ) 
end


--for subzone changes -- does not appear to be needed
--[[
function SpeakinSpell:MINIMAP_ZONE_CHANGED()
	local funcname = "MINIMAP_ZONE_CHANGED"
	self:DebugMsg(funcname, "entry")
	
	self:OnZoneChange(true)
end
--]]

--for subzone changes inside an instance
function SpeakinSpell:ZONE_CHANGED_INDOORS()
	local funcname = "ZONE_CHANGED_INDOORS"
	self:DebugMsg(funcname, "entry")
	
	self:OnZoneChange(true)
end

--for large scale zone changes
function SpeakinSpell:ZONE_CHANGED_NEW_AREA()
	local funcname = "ZONE_CHANGED_NEW_AREA"
	self:DebugMsg(funcname, "entry")
	
	self:OnZoneChange(false)
end

--covers both small and large scale zone changes?
function SpeakinSpell:ZONE_CHANGED()
	local funcname = "ZONE_CHANGED"
	self:DebugMsg(funcname, "entry")
	
	self:OnZoneChange(true)
end
